@@ -1,7 +1,7 @@
 # platform = multi_platform_all
 var_syslog_active="yes"
 
-{{% if product in ["rhel8", "fedora", "ol8", "rhv4"] %}}
+{{% if product in ["rhel8", "rhel9", "fedora", "ol8", "rhv4"] %}}
 AUDISP_SYSLOGCONFIG=/etc/audit/plugins.d/syslog.conf
 {{% else %}}
 AUDISP_SYSLOGCONFIG=/etc/audisp/plugins.d/syslog.conf
