@@ -14,8 +14,21 @@ import glob
 
 import yaml
 
-from .build_cpe import CPEDoesNotExist
-from .constants import XCCDF_REFINABLE_PROPERTIES, SCE_SYSTEM
+from .build_cpe import CPEDoesNotExist, parse_platform_definition
+from .constants import (XCCDF_REFINABLE_PROPERTIES,
+                        SCE_SYSTEM,
+                        cce_uri,
+                        dc_namespace,
+                        ocil_cs,
+                        ocil_namespace,
+                        oval_namespace,
+                        xhtml_namespace,
+                        xsi_namespace,
+                        timestamp,
+                        SSG_BENCHMARK_LATEST_URI,
+                        SSG_PROJECT_NAME,
+                        SSG_REF_URIS
+                        )
 from .rules import get_rule_dir_id, get_rule_dir_yaml, is_rule_dir
 from .rule_yaml import parse_prodtype
 
@@ -23,7 +36,7 @@ from .cce import is_cce_format_valid, is_cce_value_valid
 from .yaml import DocumentationNotComplete, open_and_expand, open_and_macro_expand
 from .utils import required_key, mkdir_p
 
-from .xml import ElementTree as ET
+from .xml import ElementTree as ET, add_xhtml_namespace, register_namespaces, parse_file
 from .shims import unicode_func
 
 
@@ -49,12 +62,13 @@ def add_sub_element(parent, tag, data):
 
     Returns the newly created subelement of type tag.
     """
+    namespaced_data = add_xhtml_namespace(data)
     # This is used because our YAML data contain XML and XHTML elements
     # ET.SubElement() escapes the < > characters by &lt; and &gt;
     # and therefore it does not add child elements
     # we need to do a hack instead
     # TODO: Remove this function after we move to Markdown everywhere in SSG
-    ustr = unicode_func("<{0}>{1}</{0}>").format(tag, data)
+    ustr = unicode_func('<{0} xmlns:xhtml="{2}">{1}</{0}>').format(tag, namespaced_data, xhtml_namespace)
 
     try:
         element = ET.fromstring(ustr.encode("utf-8"))
@@ -117,6 +131,49 @@ def check_warnings(xccdf_structure):
             raise ValueError(msg)
 
 
+def add_reference_elements(element, references, ref_uri_dict):
+    for ref_type, ref_vals in references.items():
+        for ref_val in ref_vals.split(","):
+            # This assumes that a single srg key may have items from multiple SRG types
+            if ref_type == 'srg':
+                if ref_val.startswith('SRG-OS-'):
+                    ref_href = ref_uri_dict['os-srg']
+                elif ref_val.startswith('SRG-APP-'):
+                    ref_href = ref_uri_dict['app-srg']
+                else:
+                    raise ValueError("SRG {0} doesn't have a URI defined.".format(ref_val))
+            else:
+                try:
+                    ref_href = ref_uri_dict[ref_type]
+                except KeyError as exc:
+                    msg = (
+                        "Error processing reference {0}: {1} in Rule {2}."
+                        .format(ref_type, ref_vals, self.id_))
+                    raise ValueError(msg)
+
+            ref = ET.SubElement(element, 'reference')
+            ref.set("href", ref_href)
+            ref.text = ref_val
+
+
+def add_benchmark_metadata(element, contributors_file):
+    metadata = ET.SubElement(element, "metadata")
+
+    publisher = ET.SubElement(metadata, "{%s}publisher" % dc_namespace)
+    publisher.text = SSG_PROJECT_NAME
+
+    creator = ET.SubElement(metadata, "{%s}creator" % dc_namespace)
+    creator.text = SSG_PROJECT_NAME
+
+    contrib_tree = parse_file(contributors_file)
+    for c in contrib_tree.iter('contributor'):
+        contributor = ET.SubElement(metadata, "{%s}contributor" % dc_namespace)
+        contributor.text = c.text
+
+    source = ET.SubElement(metadata, "{%s}source" % dc_namespace)
+    source.text = SSG_BENCHMARK_LATEST_URI
+
+
 class SelectionHandler(object):
     def __init__(self):
         self.refine_rules = defaultdict(list)
@@ -741,7 +798,7 @@ class Benchmark(XCCDFEntity):
         front_matter=lambda: "",
         rear_matter=lambda: "",
         cpes=lambda: list(),
-        version=lambda: "0",
+        version=lambda: "",
         profiles=lambda: list(),
         values=lambda: dict(),
         groups=lambda: dict(),
@@ -756,20 +813,10 @@ class Benchmark(XCCDFEntity):
         "description",
         "front_matter",
         "rear_matter",
-        "version",
     }
 
     GENERIC_FILENAME = "benchmark.yml"
 
-    def add_value_needed_for_ocil_clauses(self):
-        conditional_clause = Value("conditional_clause")
-        conditional_clause.title = "A conditional clause for check statements."
-        conditional_clause.description = conditional_clause.title
-        conditional_clause.type = "string"
-        conditional_clause.options = {"": "This is a placeholder"}
-
-        self.add_value(conditional_clause)
-
     def load_entities(self, rules_by_id, values_by_id, groups_by_id):
         for rid, val in self.rules.items():
             if not val:
@@ -801,8 +848,6 @@ class Benchmark(XCCDFEntity):
         data["notice_description"] = required_key(notice_contents, "description")
         del notice_contents["description"]
 
-        data["version"] = str(data["version"])
-
         return data
 
     def represent_as_dict(self):
@@ -815,12 +860,16 @@ class Benchmark(XCCDFEntity):
         return data
 
     @classmethod
-    def from_yaml(cls, yaml_file, env_yaml=None, benchmark_id="product-name"):
+    def from_yaml(cls, yaml_file, env_yaml=None):
         benchmark = super(Benchmark, cls).from_yaml(yaml_file, env_yaml)
         if env_yaml:
             benchmark.product_cpe_names = env_yaml["product_cpes"].get_product_cpe_names()
-
-        benchmark.id_ = benchmark_id
+            benchmark.cpe_platform_spec = env_yaml["product_cpes"].cpe_platform_specification
+            benchmark.id_ = env_yaml["benchmark_id"]
+            benchmark.version = env_yaml["ssg_version_str"]
+        else:
+            benchmark.id_ = "product-name"
+            benchmark.version = "0.0"
 
         return benchmark
 
@@ -852,12 +901,11 @@ class Benchmark(XCCDFEntity):
 
             self.profiles.append(new_profile)
 
-    def to_xml_element(self):
+    def to_xml_element(self, env_yaml=None):
         root = ET.Element('Benchmark')
+        root.set('id', self.id_)
+        root.set('xmlns', "http://checklists.nist.gov/xccdf/1.1")
         root.set('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance')
-        root.set('xmlns:xhtml', 'http://www.w3.org/1999/xhtml')
-        root.set('xmlns:dc', 'http://purl.org/dc/elements/1.1/')
-        root.set('id', 'product-name')
         root.set('xsi:schemaLocation',
                  'http://checklists.nist.gov/xccdf/1.1 xccdf-1.1.4.xsd')
         root.set('style', 'SCAP_1.1')
@@ -872,6 +920,9 @@ class Benchmark(XCCDFEntity):
         notice.set('id', self.notice_id)
         add_sub_element(root, "front-matter", self.front_matter)
         add_sub_element(root, "rear-matter", self.rear_matter)
+        # if there are no platforms, do not output platform-specification at all
+        if len(self.cpe_platform_spec.platforms) > 0:
+            root.append(self.cpe_platform_spec.to_xml_element())
 
         # The Benchmark applicability is determined by the CPEs
         # defined in the product.yml
@@ -881,7 +932,10 @@ class Benchmark(XCCDFEntity):
 
         version = ET.SubElement(root, 'version')
         version.text = self.version
-        ET.SubElement(root, "metadata")
+        version.set('update', SSG_BENCHMARK_LATEST_URI)
+
+        contributors_file = os.path.join(os.path.dirname(__file__), "../Contributors.xml")
+        add_benchmark_metadata(root, contributors_file)
 
         for profile in self.profiles:
             root.append(profile.to_xml_element())
@@ -898,15 +952,15 @@ class Benchmark(XCCDFEntity):
             group = self.groups.get(group_id)
             # Products using application benchmark don't have system or services group
             if group is not None:
-                root.append(group.to_xml_element())
+                root.append(group.to_xml_element(env_yaml))
 
         for rule in self.rules.values():
-            root.append(rule.to_xml_element())
+            root.append(rule.to_xml_element(env_yaml))
 
         return root
 
-    def to_file(self, file_name, ):
-        root = self.to_xml_element()
+    def to_file(self, file_name, env_yaml=None):
+        root = self.to_xml_element(env_yaml)
         tree = ET.ElementTree(root)
         tree.write(file_name)
 
@@ -942,6 +996,7 @@ class Group(XCCDFEntity):
     """
     ATTRIBUTES_TO_PASS_ON = (
         "platforms",
+        "cpe_platform_names",
     )
 
     GENERIC_FILENAME = "group.yml"
@@ -958,7 +1013,7 @@ class Group(XCCDFEntity):
         rules=lambda: dict(),
         platform=lambda: "",
         platforms=lambda: set(),
-        cpe_names=lambda: set(),
+        cpe_platform_names=lambda: set(),
         ** XCCDFEntity.KEYS
     )
 
@@ -968,7 +1023,6 @@ class Group(XCCDFEntity):
         "description",
         "front_matter",
         "rear_matter",
-        "version",
     }
 
     @classmethod
@@ -988,6 +1042,14 @@ class Group(XCCDFEntity):
 
         if data["platform"]:
             data["platforms"].add(data["platform"])
+
+        # parse platform definition and get CPEAL platform
+        if data["platforms"]:
+            for platform in data["platforms"]:
+                cpe_platform = parse_platform_definition(platform, env_yaml["product_cpes"])
+                data["cpe_platform_names"].add(cpe_platform.id)
+                # add platform to platform specification
+                env_yaml["product_cpes"].cpe_platform_specification.add_platform(cpe_platform)
         return data
 
     def load_entities(self, rules_by_id, values_by_id, groups_by_id):
@@ -999,9 +1061,15 @@ class Group(XCCDFEntity):
             if not val:
                 self.values[vid] = values_by_id[vid]
 
-        for gid, val in self.groups.items():
+        for gid in list(self.groups):
+            val = self.groups.get(gid, None)
             if not val:
-                self.groups[gid] = groups_by_id[gid]
+                try:
+                    self.groups[gid] = groups_by_id[gid]
+                except KeyError:
+                    # Add only the groups we have compiled and loaded
+                    del self.groups[gid]
+                    pass
 
     def represent_as_dict(self):
         yaml_contents = super(Group, self).represent_as_dict()
@@ -1024,21 +1092,23 @@ class Group(XCCDFEntity):
                     .format(prodtype=self.prodtype, yaml_file=yaml_file))
                 raise ValueError(msg)
 
-    def to_xml_element(self):
+    def to_xml_element(self, env_yaml=None):
         group = ET.Element('Group')
         group.set('id', self.id_)
-        if self.prodtype != "all":
-            group.set("prodtype", self.prodtype)
         title = ET.SubElement(group, 'title')
         title.text = self.title
         add_sub_element(group, 'description', self.description)
         add_warning_elements(group, self.warnings)
-        add_nondata_subelements(group, "requires", "id", self.requires)
-        add_nondata_subelements(group, "conflicts", "id", self.conflicts)
 
-        for cpe_name in self.cpe_names:
+        # This is where references should be put if there are any
+        # This is where rationale should be put if there are any
+
+        for cpe_platform_name in self.cpe_platform_names:
             platform_el = ET.SubElement(group, "platform")
-            platform_el.set("idref", cpe_name)
+            platform_el.set("idref", "#"+cpe_platform_name)
+
+        add_nondata_subelements(group, "requires", "idref", self.requires)
+        add_nondata_subelements(group, "conflicts", "idref", self.conflicts)
 
         for _value in self.values.values():
             group.append(_value.to_xml_element())
@@ -1059,7 +1129,7 @@ class Group(XCCDFEntity):
         # Add rules in priority order, first all packages installed, then removed,
         # followed by services enabled, then disabled
         for rule_id in rules_in_group:
-            group.append(self.rules.get(rule_id).to_xml_element())
+            group.append(self.rules.get(rule_id).to_xml_element(env_yaml))
 
         # Add the sub groups after any current level group rules.
         # As package installed/removed and service enabled/disabled rules are usuallly in
@@ -1098,7 +1168,7 @@ class Group(XCCDFEntity):
         groups_in_group = reorder_according_to_ordering(groups_in_group, priority_order)
         for group_id in groups_in_group:
             _group = self.groups[group_id]
-            group.append(_group.to_xml_element())
+            group.append(_group.to_xml_element(env_yaml))
 
         return group
 
@@ -1123,11 +1193,11 @@ class Group(XCCDFEntity):
         # Once the group has inherited properties, update cpe_names
         if env_yaml:
             for platform in group.platforms:
-                try:
-                    group.cpe_names.add(env_yaml["product_cpes"].get_cpe_name(platform))
-                except CPEDoesNotExist:
-                    print("Unsupported platform '%s' in group '%s'." % (platform, group.id_))
-                    raise
+                cpe_platform = parse_platform_definition(
+                    platform, env_yaml["product_cpes"])
+                group.cpe_platform_names.add(cpe_platform.id)
+                env_yaml["product_cpes"].cpe_platform_specification.add_platform(
+                    cpe_platform)
 
     def _pass_our_properties_on_to(self, obj):
         for attr in self.ATTRIBUTES_TO_PASS_ON:
@@ -1142,14 +1212,14 @@ class Group(XCCDFEntity):
         self.rules[rule.id_] = rule
         self._pass_our_properties_on_to(rule)
 
-        # Once the rule has inherited properties, update cpe_names
+        # Once the rule has inherited properties, update cpe_platform_names
         if env_yaml:
             for platform in rule.platforms:
-                try:
-                    rule.cpe_names.add(env_yaml["product_cpes"].get_cpe_name(platform))
-                except CPEDoesNotExist:
-                    print("Unsupported platform '%s' in rule '%s'." % (platform, rule.id_))
-                    raise
+                cpe_platform = parse_platform_definition(
+                    platform, env_yaml["product_cpes"])
+                rule.cpe_platform_names.add(cpe_platform.id)
+                env_yaml["product_cpes"].cpe_platform_specification.add_platform(
+                    cpe_platform)
 
     def __str__(self):
         return self.id_
@@ -1183,14 +1253,16 @@ class Rule(XCCDFEntity):
         ocil_clause=lambda: None,
         ocil=lambda: None,
         oval_external_content=lambda: None,
+        fix=lambda: "",
         warnings=lambda: list(),
         conflicts=lambda: list(),
         requires=lambda: list(),
         platform=lambda: None,
         platforms=lambda: set(),
+        sce_metadata=lambda: dict(),
         inherited_platforms=lambda: list(),
         template=lambda: None,
-        cpe_names=lambda: set(),
+        cpe_platform_names=lambda: set(),
         ** XCCDFEntity.KEYS
     )
 
@@ -1246,12 +1318,15 @@ class Rule(XCCDFEntity):
         if (
                 env_yaml and env_yaml["product"] in parse_prodtype(rule.prodtype)
                 or env_yaml and rule.prodtype == "all"):
+            # parse platform definition and get CPEAL platform
             for platform in rule.platforms:
-                try:
-                    rule.cpe_names.add(env_yaml["product_cpes"].get_cpe_name(platform))
-                except CPEDoesNotExist:
-                    print("Unsupported platform '%s' in rule '%s'." % (platform, rule.id_))
-                    raise
+                cpe_platform = parse_platform_definition(
+                    platform, env_yaml["product_cpes"])
+                rule.cpe_platform_names.add(cpe_platform.id)
+                # add platform to platform specification
+                env_yaml["product_cpes"].cpe_platform_specification.add_platform(
+                    cpe_platform)
+
 
         if sce_metadata and rule.id_ in sce_metadata:
             rule.sce_metadata = sce_metadata[rule.id_]
@@ -1445,49 +1520,35 @@ class Rule(XCCDFEntity):
                     .format(prodtype=self.prodtype, yaml_file=yaml_file))
                 raise ValueError(msg)
 
-    def to_xml_element(self):
+    def to_xml_element(self, env_yaml=None):
         rule = ET.Element('Rule')
+        rule.set('selected', 'false')
         rule.set('id', self.id_)
-        if self.prodtype != "all":
-            rule.set("prodtype", self.prodtype)
         rule.set('severity', self.severity)
         add_sub_element(rule, 'title', self.title)
         add_sub_element(rule, 'description', self.description)
+        add_warning_elements(rule, self.warnings)
+
+        if env_yaml:
+            ref_uri_dict = env_yaml['reference_uris']
+        else:
+            ref_uri_dict = SSG_REF_URIS
+        add_reference_elements(rule, self.references, ref_uri_dict)
+
         add_sub_element(rule, 'rationale', self.rationale)
 
-        main_ident = ET.Element('ident')
+        for cpe_platform_name in sorted(self.cpe_platform_names):
+            platform_el = ET.SubElement(rule, "platform")
+            platform_el.set("idref", "#"+cpe_platform_name)
+
+        add_nondata_subelements(rule, "requires", "idref", self.requires)
+        add_nondata_subelements(rule, "conflicts", "idref", self.conflicts)
+
         for ident_type, ident_val in self.identifiers.items():
-            # This is not true if items were normalized
-            if '@' in ident_type:
-                # the ident is applicable only on some product
-                # format : 'policy@product', eg. 'stigid@product'
-                # for them, we create a separate <ref> element
-                policy, product = ident_type.split('@')
-                ident = ET.SubElement(rule, 'ident')
-                ident.set(policy, ident_val)
-                ident.set('prodtype', product)
-            else:
-                main_ident.set(ident_type, ident_val)
-
-        if main_ident.attrib:
-            rule.append(main_ident)
-
-        main_ref = ET.Element('ref')
-        for ref_type, ref_val in self.references.items():
-            # This is not true if items were normalized
-            if '@' in ref_type:
-                # the reference is applicable only on some product
-                # format : 'policy@product', eg. 'stigid@product'
-                # for them, we create a separate <ref> element
-                policy, product = ref_type.split('@')
-                ref = ET.SubElement(rule, 'ref')
-                ref.set(policy, ref_val)
-                ref.set('prodtype', product)
-            else:
-                main_ref.set(ref_type, ref_val)
-
-        if main_ref.attrib:
-            rule.append(main_ref)
+            ident = ET.SubElement(rule, 'ident')
+            if ident_type == 'cce':
+                ident.set('system', cce_uri)
+                ident.text = ident_val
 
         ocil_parent = rule
         check_parent = rule
@@ -1548,30 +1609,27 @@ class Rule(XCCDFEntity):
             href = self.sce_metadata['relative_path']
             check_ref.set("href", href)
 
+        check = ET.SubElement(check_parent, 'check')
+        check.set("system", oval_namespace)
+        check_content_ref = ET.SubElement(check, "check-content-ref")
         if self.oval_external_content:
-            check = ET.SubElement(check_parent, 'check')
-            check.set("system", "http://oval.mitre.org/XMLSchema/oval-definitions-5")
-            external_content = ET.SubElement(check, "check-content-ref")
-            external_content.set("href", self.oval_external_content)
+            check_content_ref.set("href", self.oval_external_content)
         else:
             # TODO: This is pretty much a hack, oval ID will be the same as rule ID
             #       and we don't want the developers to have to keep them in sync.
             #       Therefore let's just add an OVAL ref of that ID.
-            oval_ref = ET.SubElement(check_parent, "oval")
-            oval_ref.set("id", self.id_)
+            # TODO  Can we not add the check element if the rule doesn't have an OVAL check?
+            #       At the moment, the check elements of rules without OVAL are removed by
+            #       relabel_ids.py
+            check_content_ref.set("href", "oval-unlinked.xml")
+            check_content_ref.set("name", self.id_)
 
         if self.ocil or self.ocil_clause:
-            ocil = add_sub_element(ocil_parent, 'ocil', self.ocil if self.ocil else "")
-            if self.ocil_clause:
-                ocil.set("clause", self.ocil_clause)
-
-        add_warning_elements(rule, self.warnings)
-        add_nondata_subelements(rule, "requires", "id", self.requires)
-        add_nondata_subelements(rule, "conflicts", "id", self.conflicts)
-
-        for cpe_name in self.cpe_names:
-            platform_el = ET.SubElement(rule, "platform")
-            platform_el.set("idref", cpe_name)
+            ocil_check = ET.SubElement(check_parent, "check")
+            ocil_check.set("system", ocil_cs)
+            ocil_check_ref = ET.SubElement(ocil_check, "check-content-ref")
+            ocil_check_ref.set("href", "ocil-unlinked.xml")
+            ocil_check_ref.set("name", self.id_ + "_ocil")
 
         return rule
 
@@ -1580,6 +1638,63 @@ class Rule(XCCDFEntity):
         tree = ET.ElementTree(root)
         tree.write(file_name)
 
+    def to_ocil(self):
+        if not self.ocil and not self.ocil_clause:
+            raise ValueError("Rule {0} doesn't have OCIL".format(self.id_))
+        # Create <questionnaire> for the rule
+        questionnaire = ET.Element("questionnaire", id=self.id_ + "_ocil")
+        title = ET.SubElement(questionnaire, "title")
+        title.text = self.title
+        actions = ET.SubElement(questionnaire, "actions")
+        test_action_ref = ET.SubElement(actions, "test_action_ref")
+        test_action_ref.text = self.id_ + "_action"
+        # Create <boolean_question_test_action> for the rule
+        action = ET.Element(
+            "boolean_question_test_action",
+            id=self.id_ + "_action",
+            question_ref=self.id_ + "_question")
+        when_true = ET.SubElement(action, "when_true")
+        result = ET.SubElement(when_true, "result")
+        result.text = "PASS"
+        when_true = ET.SubElement(action, "when_false")
+        result = ET.SubElement(when_true, "result")
+        result.text = "FAIL"
+        # Create <boolean_question>
+        boolean_question = ET.Element(
+            "boolean_question", id=self.id_ + "_question")
+        # TODO: The contents of <question_text> element used to be broken in
+        # the legacy XSLT implementation. The following code contains hacks
+        # to get the same results as in the legacy XSLT implementation.
+        # This enabled us a smooth transition to new OCIL generator
+        # without a need to mass-edit rule YAML files.
+        # We need to solve:
+        # TODO: using variables (aka XCCDF Values) in OCIL content
+        # TODO: using HTML formating tags eg. <pre> in OCIL content
+        #
+        # The "ocil" key in compiled rules contains HTML and XML elements
+        # but OCIL question texts shouldn't contain HTML or XML elements,
+        # therefore removing them.
+        if self.ocil is not None:
+            ocil_without_tags = re.sub(r"</?[^>]+>", "", self.ocil)
+        else:
+            ocil_without_tags = ""
+        # The "ocil" key in compiled rules contains XML entities which would
+        # be escaped by ET.Subelement() so we need to use add_sub_element()
+        # instead because we don't want to escape them.
+        question_text = add_sub_element(
+            boolean_question, "question_text", ocil_without_tags)
+        # The "ocil_clause" key in compiled rules also contains HTML and XML
+        # elements but unlike the "ocil" we want to escape the '<' and '>'
+        # characters.
+        # The empty ocil_clause causing broken question is in line with the
+        # legacy XSLT implementation.
+        ocil_clause = self.ocil_clause if self.ocil_clause else ""
+        question_text.text = (
+            u"{0}\n      Is it the case that {1}?\n      ".format(
+                question_text.text if question_text.text is not None else "",
+                ocil_clause))
+        return (questionnaire, action, boolean_question)
+
     def __hash__(self):
         """ Controls are meant to be unique, so using the
         ID should suffice"""
@@ -1661,15 +1776,16 @@ class DirectoryLoader(object):
         # we treat benchmark as a special form of group in the following code
         if self.benchmark_file:
             group = Benchmark.from_yaml(
-                self.benchmark_file, self.env_yaml, 'product-name'
+                self.benchmark_file, self.env_yaml
             )
-            group.add_value_needed_for_ocil_clauses()
             if self.profiles_dir:
                 group.add_profiles_from_dir(self.profiles_dir, self.env_yaml)
 
         if self.group_file:
             group = Group.from_yaml(self.group_file, self.env_yaml)
-            self.all_groups[group.id_] = group
+            prodtypes = parse_prodtype(group.prodtype)
+            if "all" in prodtypes or self.product in prodtypes:
+                self.all_groups[group.id_] = group
 
         return group
 
@@ -1812,14 +1928,17 @@ class LinearLoader(object):
 
     def load_benchmark(self, directory):
         self.benchmark = Benchmark.from_yaml(
-            os.path.join(directory, "benchmark.yml"), self.env_yaml, "product-name")
-        self.benchmark.add_value_needed_for_ocil_clauses()
+            os.path.join(directory, "benchmark.yml"), self.env_yaml)
 
         self.benchmark.add_profiles_from_dir(self.resolved_profiles_dir, self.env_yaml)
 
         benchmark_first_groups = self.find_first_groups_ids(directory)
         for gid in benchmark_first_groups:
-            self.benchmark.add_group(self.groups[gid], self.env_yaml)
+            try:
+                self.benchmark.add_group(self.groups[gid], self.env_yaml)
+            except KeyError as exc:
+                # Add only the groups we have compiled and loaded
+                pass
 
     def load_compiled_content(self):
         filenames = glob.glob(os.path.join(self.resolved_rules_dir, "*.yml"))
@@ -1838,4 +1957,32 @@ class LinearLoader(object):
             g.load_entities(self.rules, self.values, self.groups)
 
     def export_benchmark_to_file(self, filename):
-        return self.benchmark.to_file(filename)
+        register_namespaces()
+        return self.benchmark.to_file(filename, self.env_yaml)
+
+    def export_ocil_to_file(self, filename):
+        root = ET.Element('ocil')
+        root.set('xmlns:xsi', xsi_namespace)
+        root.set("xmlns", ocil_namespace)
+        root.set("xmlns:xhtml", xhtml_namespace)
+        tree = ET.ElementTree(root)
+        generator = ET.SubElement(root, "generator")
+        product_name = ET.SubElement(generator, "product_name")
+        product_name.text = "build_shorthand.py from SCAP Security Guide"
+        product_version = ET.SubElement(generator, "product_version")
+        product_version.text = "ssg: " + self.env_yaml["ssg_version_str"]
+        schema_version = ET.SubElement(generator, "schema_version")
+        schema_version.text = "2.0"
+        timestamp_el = ET.SubElement(generator, "timestamp")
+        timestamp_el.text = timestamp
+        questionnaires = ET.SubElement(root, "questionnaires")
+        test_actions = ET.SubElement(root, "test_actions")
+        questions = ET.SubElement(root, "questions")
+        for rule in self.rules.values():
+            if not rule.ocil and not rule.ocil_clause:
+                continue
+            questionnaire, action, boolean_question = rule.to_ocil()
+            questionnaires.append(questionnaire)
+            test_actions.append(action)
+            questions.append(boolean_question)
+        tree.write(filename)
